//
// NTSC composite encoder module
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Version : beta1
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
// 2002. 9. 3 : T.Satoh : Created
//

module ntsc_enc(
	T4SC_I,RESET,Y,V,U,
	VPOS_O,HPOS_O,
	HSYNC_O,VSYNC_O,CSYNC_O,Y_O,C_O,V_O,
	SC_O,CB_O,WIN_O
);

input T4SC_I,RESET;
input [7:0] Y,V,U;
output HSYNC_O,VSYNC_O,CSYNC_O;
output [8:0] Y_O,C_O,V_O;

output [9:0] HPOS_O;
output [8:0] VPOS_O;

output [1:0] SC_O;
output CB_O , WIN_O;


//
// Video timming gnnerator
//
// hpos : H display position (0-909)
// vpos : V display position (0-262)
// vout : 9bit composit V signal
// cout : 9bit S1 C signal
// yout : 9bit S2 Y signal
//

// Y fixed level
`define Y_SYNC  0
`define Y_BLANK 40 //256mV

// C fixed level
`define C_ZERO    0
`define C_CB30    8  //  52mV 
`define C_CB70   16  // 100mV
`define C_CBURST 23  // 144mV

//
// V-LINE state counters
//
`define VC_PRE_EQUALIZING   3
`define VC_VERTICAL_SYNC    3
`define VC_POST_EQUALIZING  3
`define VC_PRE_BLANK       10
//
// H-LINE state counters
//
//-----------------------------------------------------------------------------
// H-LINE of display area
//
//  19clk : FrontPorch         : 256mV
//  67clk : HorizontalSync     : 0V
//   9clk : Breezeway          : 256mV
//   2clk :  33% of ColorBurst : 256mV +- 52/0mV
//   2clk :  70% of ColorBurst : 256mV +-100/0mV
//  32clk : 100% of ColorBurst : 256mV +-144/0mV
//   2clk :  70% of ColorBurst : 256mV +-100/0mV
//   2clk :  33% of ColorBurst : 256mV +- 52/0mV
//  23clk : ColorBackPorch     : 256mV
// 752clk : pixel data         : 256mV +- Y +-C  : RED == 256mV+(56mV,321mV,608mV,264mV)
//-----------------------------------------------------------------------------
//
`define HC_FRONT_PORCH    19
`define HC_HOR_SYNC      (67+`HC_FRONT_PORCH)
`define HC_BREEZE_WAY    ( 9+`HC_HOR_SYNC)
`define HC_C_BURST1      ( 2+`HC_BREEZE_WAY)
`define HC_C_BURST2      ( 2+`HC_C_BURST1)
`define HC_C_BURST3      (32+`HC_C_BURST2)
`define HC_C_BURST4      ( 2+`HC_C_BURST3)
`define HC_C_BURST5      ( 2+`HC_C_BURST4)
`define HC_C_BACK_PORCH  (23+`HC_C_BURST5)
// other == display area
`define HC_RETURN       910

//
// internal regs.
//
reg [9:0] hc;
reg [8:0] vc;
reg [1:0] sc;
//wire [9:0] hpos = hcnt - 139; // H position
//wire [8:0] vpos = vcnt - 22;  // V position

reg  [8:0] cout,yout,vout; // 9bit Y / C / composite video out

//
// V states
//
wire equalizing = vc<(`VC_POST_EQUALIZING-1);
wire vsync_d    = (vc>=(`VC_PRE_EQUALIZING-1)) && (vc<(`VC_VERTICAL_SYNC-1));
wire vblank_d   = vc<(`VC_PRE_BLANK-1);

//
// H states
//
wire hsync_d    = (hc>=(`HC_FRONT_PORCH-1)) && (hc<(`HC_HOR_SYNC-1));
wire esync_d    = equalizing && ((hc>=(455-1)) && (hc<(910-1)));
wire cburst_d   = (hc>=(`HC_C_BURST1-1)) && (hc<=(`HC_C_BURST5-1));
wire cburst30_d = (hc==(`HC_C_BURST1-1))||(hc==(`HC_C_BURST5-1));
wire cburst70_d = (hc==(`HC_C_BURST2-1))||(hc==(`HC_C_BURST4-1));
wire hblank_d   = (hc<(`HC_C_BACK_PORCH-1));

//
// compsite states
//
wire blank_d = vblank_d || hblank_d;
wire csync_d = vsync_d ^ (hsync_d | esync_d );

// COLOR
wire [7:0] v_d,u_d,c_d;
assign v_d = cburst30_d ? `C_CB30
           : cburst70_d ? `C_CB70
           : cburst_d   ? `C_CBURST
           :              -U;      // window!
assign u_d = cburst_d   ? `C_ZERO : V;
assign c_d  = blank_d & ~cburst_d ? `C_ZERO	 // blank
	        : (sc==2'b00) ?  v_d //   0'
	        : (sc==2'b01) ?  u_d //  90'
	        : (sc==2'b10) ? -v_d // 180'
	        :               -u_d;// 270'
// YSIGNAL
wire [8:0] y_d;
assign y_d = csync_d ? `Y_SYNC   // sync
	       : hblank_d ? `Y_BLANK   // blank
	       :           `Y_BLANK + Y;      // window !

// Timming Generator
//
reg Field , vsync , hsync , csync;
reg cburst , window;

//always @(posedge T4SC_I or posedge RESET)
always @(posedge T4SC_I)
begin
	if( RESET )
	begin
		// Reset
		Field <= 0;
		vc  <= 0;
		hc  <= 0;
		sc <= 0;
	end else begin

	// Color Signal

	// counter
	if(hc==`HC_RETURN)
	begin
		hc <= 0;
//		sc <= 0;
		if( vc == 262)
			vc <= 0;
		else
			vc <= vc + 1;
	end else begin
		hc <= hc+1;
	end

	// sub carrier
	sc <= sc + 1;


	//
	// OUTPUT signals
	//

	// SYNCs
	vsync <= vsync_d;
	hsync <= hsync_d;
	csync <= csync_d;

	// debug
	cburst <= cburst_d;
	window <= ~csync_d & ~blank_d;

	// Y / C / V OUT
	yout  <= y_d;
	cout  <= {c_d[7],c_d};
	vout  <= y_d + {c_d[7],c_d};

	end
end

assign HSYNC_O = hsync;
assign VSYNC_O = vsync;
assign CSYNC_O = csync;
assign Y_O = yout;
assign C_O = cout;
assign V_O = vout;
assign VPOS_O = vc - `VC_PRE_BLANK;
assign HPOS_O = hc - `HC_C_BACK_PORCH;
assign SC_O = sc;
assign CB_O = cburst;
assign WIN_O = window;

endmodule

